{% include "header.html" %}

<nav aria-label="breadcrumb">
  <ol class="breadcrumb bg-light">
    <li class="breadcrumb-item"><a href="{{ template.api_root }}?f=html{{ template.extra_params }}">Home</a></li>
    <li class="breadcrumb-item active" aria-current="page">Conformance</li>
    <li class="ms-auto json-link"><a target="_blank" href="{{ template.api_root }}/conformance?f=json{{ template.extra_params }}">JSON</a></li>
  </ol>
</nav>

<h1>{{ template.title }} - Conformance</h1>

<p>This API implements the conformance classes from standards and community specifications that are listed below.</p>

<h2>Links</h2>
<ul>
{% for url in response.conformsTo %}
  <li> <a target="_blank" href="{{ url }}">{{ url }}</a></li>
{% endfor %}
</ul>

{% include "footer.html" %}
